@@ -2,6 +2,8 @@
 
 Gio = imports.gi.Gio;
 
+JSON = imports.JSON;
+
 var redditurl = Gio.file_new_for_uri("http://reddit.com/.json");
 var reddit = JSON.parse(redditurl.read().get_contents());
 
@@ -9,11 +11,11 @@ stories = reddit.data.children;
 
 for (i in stories){
     var story = stories[i].data;
-    Seed.print("======");
-    Seed.print(story.ups + "/" +
+    print("======");
+    print(story.ups + "/" +
 	       story.downs + " " +
 	       story.title + " (" +
 	       story.domain +")" );
-    Seed.print(story.author);
+    print(story.author);
 }
 
