@@ -1,6 +1,4 @@
-/*
- * -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-
- */
+/* -*- mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 2; -*- */
 
 /*
  * This file is part of Seed, the GObject Introspection<->Javascript bindings.
@@ -16,7 +14,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with Seed.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) Robert Carr 2008 <carrr@rpi.edu>
+ * Copyright (C) Robert Carr 2009 <carrr@rpi.edu>
  */
 
 #ifndef _SEED_ENGINE_H
@@ -68,4 +66,18 @@ typedef JSObjectRef (*SeedModuleInitCallback) (SeedEngine * eng);
 
 void seed_prepare_global_context (JSContextRef ctx);
 
+SeedScript *seed_make_script (JSContextRef ctx,
+			      const gchar * js,
+			      const gchar * source_url, gint line_number);
+SeedScript *seed_script_new_from_file (JSContextRef ctx, gchar * file);
+JSValueRef seed_script_exception (SeedScript * s);
+
+JSValueRef seed_evaluate (JSContextRef ctx, SeedScript * script,
+			  JSObjectRef this);
+
+void seed_script_destroy (SeedScript * s);
+
+JSValueRef seed_simple_evaluate (JSContextRef ctx, const gchar * script,
+				 JSValueRef * exception);
+
 #endif
